import json
import logging
import threading
from typing import Any, Dict, List, Optional

from .utils import load_env_config, load_local_config

logger = logging.getLogger(__name__)


class AppConfig:
    """配置环境管理器，统一管理所有配置源"""

    def __init__(self, env_prefix: str = "", local_fallback_path: Optional[str] = "config/config.yaml", lowercase_enabled: bool = True):
        self._lock = threading.RLock()
        self.lowercase_enabled = lowercase_enabled
        self._fallback_config = (
            load_local_config(local_fallback_path) if local_fallback_path else {}
        )
        self._remote_configs: Dict[str, Dict[str, Any]] = {}
        self._env_config = load_env_config(env_prefix, lowercase_enabled)

    def get(self, key: str, default=None, case_sensitive: bool = True):
        """按优先级获取配置并合并：环境变量 > 远程配置 > 本地fallback

        Args:
            key: 配置键，支持点号分隔的嵌套键
            default: 默认值
            case_sensitive: 是否区分大小写，默认为True
        """
        # 将点号转换为双下划线，用于环境变量查找
        env_key = key.replace(".", "__").upper()
        keys = key.split(".")

        # 按优先级从低到高获取配置
        fallback_cfg = self._get_nested_config(self._fallback_config, keys, case_sensitive)
        remote_cfg = self._get_from_remote_configs(keys, case_sensitive)
        env_cfg = self._get_nested_config(self._env_config, keys, case_sensitive)
        # 如果都是非字典类型，按优先级返回
        configs = [fallback_cfg, remote_cfg, env_cfg]
        non_dict_configs = [
            cfg for cfg in configs if cfg is not None and not isinstance(cfg, dict)
        ]
        if non_dict_configs:
            return self._auto_cast(non_dict_configs[-1])  # 返回最高优先级的非字典配置

        # 如果涉及字典类型，进行合并
        result = {}
        # 按优先级从低到高合并字典配置
        for cfg in [fallback_cfg, remote_cfg, env_cfg]:
            if isinstance(cfg, dict):
                result.update(cfg)

        return result if result else default

    def _get_nested_config(self, config: Dict[str, Any], keys: List[str], case_sensitive: bool = True):
        """获取嵌套配置值"""
        if config is None:
            return None
        current = config
        for k in keys:
            if isinstance(current, dict):
                if case_sensitive:
                    current = current.get(k)
                else:
                    # 不区分大小写查找
                    current = next((v for kk, v in current.items() if kk.lower() == k.lower()), None)
                if current is None:
                    return None
            else:
                return None
        return current

    def _get_from_remote_configs(self, keys: List[str], case_sensitive: bool = True):
        """从远程配置中按优先级获取配置"""
        with self._lock:
            for config_dict in self._remote_configs.values():
                cfg = self._get_nested_config(config_dict, keys, case_sensitive)
                if cfg is not None:
                    return cfg
        return None

    # def get(self, key: str, default=None):
    #     """按优先级获取配置并合并：环境变量 > 远程配置 > 本地fallback"""
    #     keys = key.split(".")
    #
    #     # 按优先级从低到高获取配置
    #     fallback_cfg = self._get_nested_config(self._fallback_config, keys)
    #     remote_cfg = self._get_from_remote_configs(keys)
    #     env_cfg = self._get_nested_config(self._env_config, keys)
    #
    #     # 如果都是非字典类型，按优先级返回
    #     configs = [fallback_cfg, remote_cfg, env_cfg]
    #     non_dict_configs = [
    #         cfg for cfg in configs if cfg is not None and not isinstance(cfg, dict)
    #     ]
    #     if non_dict_configs:
    #         return self._auto_cast(non_dict_configs[-1])  # 返回最高优先级的非字典配置
    #
    #     # 如果涉及字典类型，进行合并
    #     result = {}
    #     # 按优先级从低到高合并字典配置
    #     for cfg in [fallback_cfg, remote_cfg, env_cfg]:
    #         if isinstance(cfg, dict):
    #             result.update(cfg)
    #
    #     return result if result else default
    #
    # def _get_from_remote_configs(self, keys: List[str]):
    #     """从远程配置中按优先级获取配置"""
    #     # 按照一定的优先级顺序遍历远程配置
    #     # 这里可以按照配置加载的顺序或者其他逻辑确定优先级
    #     with self._lock:
    #         for config_dict in self._remote_configs.values():
    #             cfg = self._get_nested_config(config_dict, keys)
    #             if cfg is not None:
    #                 return cfg
    #     return None
    #
    # def _get_nested_config(self, config: Dict[str, Any], keys: List[str]):
    #     """获取嵌套配置值"""
    #     # 判断config是否为null
    #     if config is None:
    #         return []
    #     current = config
    #     for k in keys:
    #         if isinstance(current, dict) and k in current:
    #             current = current[k]
    #         else:
    #             return None
    #     return current

    def set_remote_config(self, config: Dict[str, Any]):
        """添加远程配置"""
        with self._lock:
            self._remote_configs = config

    def _auto_cast(self, value: str) -> Any:
        """类型自动转换"""
        if isinstance(value, str):
            if value.lower() in ("true", "false"):
                return value.lower() == "true"
            if value.isdigit():
                return int(value)
            try:
                return float(value)
            except ValueError:
                pass
        return value

    def print_all_configs(self):
        """打印完整配置，按优先级顺序显示"""
        configs = [
            ("🟢 环境变量配置 (最高优先级)", self._env_config),
            ("🔵 远程配置 (Nacos等)", self._remote_configs),
            ("🟡 本地Fallback配置 (最低优先级)", self._fallback_config),
        ]

        print("\n" + "=" * 60)
        print("配置环境管理器 - 完整配置信息")
        print("=" * 60)

        for title, config_data in configs:
            print(f"\n{title}:")
            if config_data:
                print(json.dumps(config_data, indent=2, ensure_ascii=False))
            else:
                print("  (无配置数据)")

        print("\n" + "=" * 60)


# app_config = ConfigEnvironment(env_prefix="", local_fallback_path="config/config.yaml")

# 全局缓存 + 锁
_app_config: Optional[AppConfig] = None
_config_lock = threading.Lock()

def _create_config(
    env_prefix: str = "",
    local_fallback_path: str = "config/config.yaml"
) -> AppConfig:
    """工厂函数：创建新配置实例（一般只用于测试或多实例场景）"""
    return AppConfig(env_prefix, local_fallback_path)

def get_app_config() -> AppConfig:
    """获取全局唯一的配置实例（懒加载 + 线程安全）"""

    global _app_config
    if _app_config is not None:
        return _app_config
    with _config_lock:
        if _app_config is None:
            _app_config = AppConfig()  # 使用默认参数
        return _app_config


def get_config_value(
        key: str,
        default: Any = None,
        case_sensitive: bool = True
) -> Any:
    """
    快捷获取配置值。

    示例：
        host = get_config_value("database.host", default="localhost")

    注意：此函数是 get_app_config().get(...) 的便捷包装，
         所有逻辑仍由 ConfigEnvironment.get() 处理。
    """
    return get_app_config().get(key, default=default, case_sensitive=case_sensitive)

def init_app_config(
    env_prefix: str = "",
    local_fallback_path: str = "config/config.yaml",
    lowercase_enabled: bool = True
) -> AppConfig:
    """显式初始化全局配置（用于主程序入口，确保尽早加载）"""
    global _app_config
    with _config_lock:
        if _app_config is None:
            _app_config = AppConfig(env_prefix, local_fallback_path, lowercase_enabled)
        else:
            # 可选：抛出警告或允许覆盖（根据需求）
            pass
    return _app_config


import functools
import logging
from typing import Any, Dict

from .config_app import AppConfig, app_config

logger = logging.getLogger(__name__)


def from_nacos_config(config_path: str = None):
    """
    通用注解装饰器，用于从Nacos配置中实例化实体

    Args:
        config_path: 配置路径，如 "nacos.shared_configs" 或 "pgsql"
    """

    def decorator(cls):
        original_init = cls.__init__

        @functools.wraps(original_init)
        def new_init(self, config: AppConfig = app_config, *args, **kwargs):
            # 调用原始构造函数
            original_init(self, *args, **kwargs)

            # 如果提供了Nacos配置，则进行属性填充
            if config and config_path:
                config_data = config.get(config_path)
                if config_data:
                    _populate_object_from_dict(self, config_data)

        cls.__init__ = new_init
        return cls

    return decorator


def _extract_config_by_path(config: Dict[str, Any], path: str) -> Dict[str, Any]:
    """根据路径从配置字典中提取子配置"""
    keys = path.split(".")
    current = config
    current.get(path)
    return current


def _populate_object_from_dict(obj: Any, data: Dict[str, Any]):
    """将字典数据填充到对象属性中，支持嵌套对象类型"""
    for key, value in data.items():
        # 转换为Python风格的属性名（下划线命名）
        attr_name = key.replace("-", "_")
        if hasattr(obj, attr_name):
            # 正确获取类型注解
            attr_type = None
            if hasattr(obj, "__annotations__"):
                attr_type = obj.__annotations__.get(attr_name)

            # 检查是否为复杂类型
            if isinstance(value, dict) and attr_type and hasattr(attr_type, "__call__"):
                # 处理嵌套对象
                nested_obj = getattr(obj, attr_name)
                if nested_obj is None:
                    # 如果属性为空，尝试创建新实例
                    try:
                        # 确保类型是可调用的（类）
                        if callable(attr_type):
                            nested_obj = attr_type()
                            setattr(obj, attr_name, nested_obj)
                            _populate_object_from_dict(nested_obj, value)
                    except Exception as e:
                        print(f"Failed to create nested object for {attr_name}: {e}")
                        continue
                else:
                    _populate_object_from_dict(nested_obj, value)
            elif isinstance(value, list) and len(value) > 0:
                # 处理列表类型
                if isinstance(value[0], dict):
                    item_type = _get_list_item_type(obj, attr_name)
                    if item_type and callable(item_type):
                        nested_list = []
                        for item_data in value:
                            try:
                                item_obj = item_type()
                                _populate_object_from_dict(item_obj, item_data)
                                nested_list.append(item_obj)
                            except Exception as e:
                                print(
                                    f"Failed to create list item for {attr_name}: {e}"
                                )
                                continue
                        setattr(obj, attr_name, nested_list)
                else:
                    # 简单列表类型直接赋值
                    setattr(obj, attr_name, value)
            else:
                # 简单类型直接赋值
                setattr(obj, attr_name, value)


def _get_list_item_type(obj: Any, attr_name: str):
    """获取列表属性的元素类型"""
    try:
        # 通过类型注解获取元素类型
        if hasattr(obj, "__annotations__"):
            annotations = obj.__annotations__
            attr_type = annotations.get(attr_name)
            if hasattr(attr_type, "__args__") and len(attr_type.__args__) > 0:
                return attr_type.__args__[0]
    except Exception:
        pass
    return None



from typing import List, Optional

from pydantic import BaseModel

class SharedConfigEntity(BaseModel):
    data_id: str = ""
    group: str = ""
    refresh: bool = True


class NacosConfigEntity(BaseModel):
    service_host: Optional[str] = None
    server_addr: str = ""
    namespace: str = ""
    username: str = "nacos"
    password: str = "nacos"
    config_data_id: str = ""
    config_group: str = "DEFAULT_GROUP"
    shared_configs: List[SharedConfigEntity] = []
    metadata: dict = []



"""
定义一个常量类
"""

from typing import ClassVar


class NacosConstant:
    """
    nacos配置相关常量
    """

    NACOS_SERVER_ADDRESSES: ClassVar[str] = "nacos.server_addr"
    NACOS_CONFIG_DATA_ID: ClassVar[str] = "nacos.config_data_id"
    NACOS_CONFIG_GROUP: ClassVar[str] = "nacos.config_group"
    NACOS_CONFIG_NAMESPACE: ClassVar[str] = "nacos.namespace"
    NACOS_CONFIG_USERNAME: ClassVar[str] = "nacos.username"
    NACOS_CONFIG_PASSWORD: ClassVar[str] = "nacos.password"

    """
    nacos服务注册相关常量
    """
    NACOS_SERVICE_NAME: ClassVar[str] = "app.name"
    NACOS_SERVICE_PORT: ClassVar[int] = "app.port"
    NACOS_SERVICE_HOST: ClassVar[str | None] = "nacos.service_host"
    NACOS_SERVICE_METADATA: ClassVar[dict | None] = "nacos.metadata"



# config.py

import json
import logging
import threading
from typing import Any, ClassVar, Dict, Optional

import yaml
from nacos import NacosClient

from .config_app import AppConfig,init_app_config
from .entity import SharedConfigEntity

logger = logging.getLogger(__name__)


class NacosConfig:
    """Nacos配置管理器"""

    # 类级单例（线程安全）
    _instance: ClassVar[Optional["NacosConfig"]] = None
    _instance_lock = threading.Lock()

    def __init__(
        self,
        client: NacosClient,
        data_id: str,
        group: str = "DEFAULT_GROUP",
        config_env: AppConfig = None,
        shared_configs: list[SharedConfigEntity] = [],
    ):
        self.client = client
        self.data_id = data_id
        self.group = group
        self.config_env = config_env or init_app_config()
        self._lock = threading.RLock()
        self.shared_configs = shared_configs
        self._nacos_config: Dict[str, Any] = {}
        self._load_and_watch(self.data_id, self.group)

        for shared_config in shared_configs:
            self._load_and_watch(shared_config.data_id, shared_config.group)

        # 将Nacos配置注入到统一配置环境
        self.config_env.set_remote_config(self._nacos_config)

        # 打印app_config配置
        logger.info(self.config_env.print_all_configs())

        # 注册单例
        if NacosConfig._instance is None:
            with NacosConfig._instance_lock:
                if NacosConfig._instance is None:
                    NacosConfig._instance = self
                    logger.info(
                        "Intialized NacosConfig with 3-level fallback: env > nacos > local"
                    )

    @classmethod
    def get_instance(cls) -> "NacosConfig":
        """供非请求上下文使用（如工具函数、service）"""
        if cls._instance is None:
            raise RuntimeError(
                "NacosConfig not initialized. "
                "Ensure init_nacos() is called during app startup."
            )
        return cls._instance

    def _detect_format(self) -> str:
        """根据 data_id 后缀判断格式"""
        if self.data_id.endswith((".yaml", ".yml")):
            return "yaml"
        elif self.data_id.endswith(".json"):
            return "json"
        else:
            # 默认尝试 YAML（因 YAML 是 JSON 超集）
            return "yaml"

    def _detect_format_by_data_id(self, data_id: str) -> str:
        """根据 data_id 后缀判断格式"""
        if data_id.endswith((".yaml", ".yml")):
            return "yaml"
        elif data_id.endswith(".json"):
            return "json"
        else:
            # 默认尝试 YAML（因 YAML 是 JSON 超集）
            return "yaml"

    def _parse_content(self, content: str, fmt: str) -> Dict[str, Any]:
        """安全解析配置内容"""
        if not content:
            return {}
        content = content.strip()
        if not content:
            return {}

        try:
            if fmt == "json":
                return json.loads(content)
            else:  # yaml or auto
                # 使用 yaml.safe_load，它能解析 JSON 和 YAML
                parsed = yaml.safe_load(content)
                # yaml.safe_load 可能返回非 dict（如纯字符串），需校验
                if parsed is None:
                    return {}
                if not isinstance(parsed, dict):
                    logger.warning(
                        f"Config {self.data_id} parsed as non-dict: {type(parsed)}"
                    )
                    return {}
                return parsed
        except (yaml.YAMLError, json.JSONDecodeError) as e:
            logger.error(f"Failed to parse {fmt.upper()} config '{self.data_id}': {e}")
            logger.debug(f"Problematic content:\n{content[:500]}...")
            raise

    def _load_and_watch(self, data_id: str = None, group: str = None):
        """首次加载 + 注册监听"""
        fmt = self._detect_format()
        try:
            raw = self._load(data_id, group)
            logger.info(f"Loaded config '{self.data_id}' (format: {fmt})")
        except Exception as e:
            logger.error(f"Failed to load initial config '{data_id}': {e}")
            self._nacos_config = {}  # 初始化为空 dict，避免后续 get 报错

        # 注册监听器
        self._watch(data_id, group, self._on_change_callback)

    def _load(self, data_id, group):
        raw = self.client.get_config(data_id, group)
        fmt = self._detect_format_by_data_id(data_id)
        parsed = self._parse_content(raw, fmt)
        self._nacos_config[f"{group}/{data_id}"] = parsed
        return parsed

    def _watch(self, data_id, group, callback):
        def wrapper(args):
            fmt = self._detect_format_by_data_id(data_id)
            new_config = self._parse_content(args["content"], fmt)
            # self._nacos_config[f"{group}/{data_id}"] = new_config
            callback(new_config, fmt, data_id, group)

        try:
            self.client.add_config_watcher(data_id, group, wrapper)
            logger.info(f"Started watching config '{data_id}' in group '{group}'")
        except Exception as e:
            logger.error(f"Failed to watch config '{data_id}' in group '{group}': {e}")

    def _on_change_callback(self, new_config: Dict[str, Any], fmt: str, data_id, group):
        """配置变更回调"""
        try:
            config_key = f"{group}/{data_id}"

            with self._lock:
                if config_key in self._nacos_config:
                    old_config = self._nacos_config[config_key]
                    # 使用深度比较，获取完整路径
                    changes = self._deep_diff(old_config, new_config)

                    # 更新配置
                    self._nacos_config[config_key] = new_config

                    # 记录变更详情
                    change_details = []
                    if changes["added"]:
                        added_info = []
                        for path, value in changes["added"].items():
                            added_info.append(f"{path}={value}")
                        change_details.append(f"added: {added_info}")

                    if changes["removed"]:
                        removed_info = []
                        for path, value in changes["removed"].items():
                            removed_info.append(f"{path}={value}")
                        change_details.append(f"removed: {removed_info}")

                    if changes["modified"]:
                        modified_info = []
                        for path, values in changes["modified"].items():
                            modified_info.append(
                                f"{path}: {values['old']} -> {values['new']}"
                            )
                        change_details.append(f"modified: {modified_info}")

                    if change_details:
                        logger.info(
                            f"Config '{data_id}' in group '{group}' updated (format: {fmt})"
                            + f" — {', '.join(change_details)}"
                        )
                    else:
                        logger.info(
                            f"Config '{data_id}' in group '{group}' updated (no changes detected)"
                        )
                else:
                    # 新配置
                    self._nacos_config[config_key] = new_config
                    logger.info(
                        f"New config '{data_id}' in group '{group}' loaded (format: {fmt})"
                    )

        except Exception as e:
            logger.error(f"Config update failed, keeping old config: {e}")

    def _deep_diff(
        self, old_dict: Dict[str, Any], new_dict: Dict[str, Any], prefix: str = ""
    ) -> Dict[str, Any]:
        """深度比较两个字典的差异，返回完整的变更路径和具体值"""
        changes = {"added": {}, "removed": {}, "modified": {}}

        # 获取所有键
        all_keys = set(old_dict.keys()) | set(new_dict.keys())

        for key in all_keys:
            current_path = f"{prefix}.{key}" if prefix else key

            if key not in old_dict:
                changes["added"][current_path] = new_dict[key]
            elif key not in new_dict:
                changes["removed"][current_path] = old_dict[key]
            else:
                old_val = old_dict[key]
                new_val = new_dict[key]

                # 如果都是字典，递归比较
                if isinstance(old_val, dict) and isinstance(new_val, dict):
                    nested_changes = self._deep_diff(old_val, new_val, current_path)
                    # 合并嵌套变更
                    for change_type in ["added", "removed", "modified"]:
                        if nested_changes[change_type]:
                            changes[change_type].update(nested_changes[change_type])
                # 如果值不同
                elif old_val != new_val:
                    changes["modified"][current_path] = {"old": old_val, "new": new_val}

        return changes

    def get(self, key: str, default=None):
        """通过配置环境获取配置"""
        return self.config_env.get(key, default)



# nacos/__init__.py
import logging
from typing import Optional, Tuple

from .client import NacosClientManager
from .config_app import init_app_config, get_app_config, get_config_value
from .config import NacosConfig
from .constant import NacosConstant
from .discovery import ServiceDiscovery
from .entity import NacosConfigEntity, SharedConfigEntity

logger = logging.getLogger(__name__)

__all__ = ["init_nacos_client", "get_service_discovery", "NacosConstant", "get_config_value", "get_app_config"]


# 全局变量（由 init_nacos 设置）
_nacos_manager: Optional[NacosClientManager] = None


def init_nacos_client(local_fallback_path: str = "config/config.yaml",
    lowercase_enabled: bool = True):
    init_app_config(local_fallback_path=local_fallback_path, lowercase_enabled=lowercase_enabled)
    nacos_config_data = get_config_value("nacos", {})
    nacos_entity = NacosConfigEntity(**nacos_config_data)
    """
    初始化 Nacos 客户端
    """
    return init_nacos(
        server_addresses=nacos_entity.server_addr,
        namespace=nacos_entity.namespace,
        username=nacos_entity.username,
        password=nacos_entity.password,
        service_name=get_config_value(key=NacosConstant.NACOS_SERVICE_NAME, case_sensitive= False),
        service_port=get_config_value(key=NacosConstant.NACOS_SERVICE_PORT, case_sensitive= False),
        config_data_id=nacos_entity.config_data_id,
        config_group=nacos_entity.config_group,
        shared_configs=nacos_entity.shared_configs,
        metadata=nacos_entity.metadata,
    )


def init_nacos(
    server_addresses: str,
    namespace: str = "",
    username: str = "",
    password: str = "",
    service_name: str = "sv-maas-service",
    service_port: int = 8000,
    service_host: str | None = None,
    config_data_id: str = "sv-maas-service-prod.yaml",
    config_group: str = "DEFAULT_GROUP",
    shared_configs: list[SharedConfigEntity] = [],
    metadata: dict | None = None,
) -> Tuple[NacosClientManager, NacosConfig]:
    """
    初始化 Nacos：客户端 + 配置 + 服务注册
    返回 (client, config)，供依赖注入或全局使用
    """
    global _nacos_manager

    # 1. 创建客户端
    nacos_manager = NacosClientManager(
        server_addresses=server_addresses,
        namespace=namespace,
        username=username,
        password=password,
    )

    _nacos_manager = nacos_manager

    # 2. 加载配置
    config = nacos_manager.init_config(
        data_id=config_data_id,
        group=config_group,
        config_env=get_app_config(),
        shared_configs=shared_configs,
    )

    # 3. 注册服务
    nacos_manager.register_service(
        service_name=service_name,
        port=service_port,
        ip=service_host or None,
        group=config_group,
        metadata=metadata or {},
        heartbeat_interval=5,
    )
    logger.info("Application started with Nacos (env > nacos > local)")

    return nacos_manager, config


def get_nacos_config(key: str, default=None):
    return NacosConfig.get_instance().get(key, default)


def get_service_discovery() -> ServiceDiscovery:
    if _nacos_manager is None:
        raise RuntimeError("Nacos not initialized. Call init_nacos() in startup.")
    return _nacos_manager.get_discovery()



# utils.py
import logging
import os
import re
import threading
import socket
from typing import Any, Dict

import yaml

logger = logging.getLogger(__name__)

def get_local_ip():
    # 1. 优先从环境变量获取（K8s / Docker 推荐）
    ip = os.getenv("POD_IP") or os.getenv("HOST_IP")
    if ip:
        return ip

    # 2. 尝试通过连接一个本地保留地址（不发包！）
    try:
        with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
            # 连接到一个不可能出去的地址，但会绑定本地出口 IP
            s.connect(("10.255.255.255", 1))
            ip = s.getsockname()[0]
        return ip
    except Exception:
        pass

    # 3. 最后 fallback 到 hostname 解析
    try:
        hostname = socket.gethostname()
        ip_list = socket.gethostbyname_ex(hostname)[2]
        for ip in ip_list:
            if not ip.startswith(("127.", "169.254.")):
                return ip
    except Exception:
        pass

    return "127.0.0.1"

def load_local_config(local_config_path: str = os.path.join("config", "config.yaml")):
    """加载本地配置文件"""
    local_config: Dict[str, Any] = {}
    try:
        with open(local_config_path, "r", encoding="utf-8") as f:
            local_config = yaml.safe_load(f) or {}
        logger.info(f"Loaded local config from {local_config_path}")
    except Exception as e:
        logger.warning(f"Failed to load local config: {e}")
    finally:
        return local_config


def load_env_config(prefix: str = "", lowercase_enabled: bool = True) -> Dict[str, Any]:
    """
    从环境变量加载配置，支持嵌套。
    示例：
      MODEL__TEMPERATURE=0.9 → {"model": {"temperature": "0.9"}}
      DATABASE__PORT=5432     → {"database": {"port": "5432"}}
    """
    env_config: Dict[str, Any] = {}
    pattern = re.compile(r"^[A-Z_][A-Z0-9_]*$")

    for key, value in os.environ.items():
        if prefix and not key.startswith(prefix):
            continue
        # clean_key = key[len(prefix) :] if prefix else key
        clean_key = key
        if not pattern.match(clean_key):
            continue


        # 将 A__B__C 转为 ["a", "b", "c"]
        parts = [part.lower() if lowercase_enabled else part
                 for part in clean_key.split("__") if part]
        if not parts:
            continue

        # 构建嵌套字典
        current = env_config
        for part in parts[:-1]:
            if part not in current:
                current[part] = {}
            current = current[part]
        current[parts[-1]] = value

    return env_config


class SafeTimer:
    """安全的定时器，支持取消"""

    def __init__(self, interval: float, function, *args, **kwargs):
        self.interval = interval
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.timer = None
        self._lock = threading.Lock()

    def start(self):
        with self._lock:
            if self.timer is None or not self.timer.is_alive():
                self.timer = threading.Timer(self.interval, self._run)
                self.timer.daemon = True
                self.timer.start()

    def _run(self):
        try:
            self.function(*self.args, **self.kwargs)
        finally:
            self.start()  # 循环执行

    def cancel(self):
        with self._lock:
            if self.timer:
                self.timer.cancel()
                self.timer = None


def clean_dict_v1(data: dict) -> dict:
    """清除字典中值为None的字段"""
    return {k: v for k, v in data.items() if v is not None}



# registry.py
import logging
import threading
from typing import Optional

from nacos import NacosClient

from .utils import SafeTimer, get_local_ip

logger = logging.getLogger(__name__)


class ServiceRegistry:
    def __init__(
        self,
        client: NacosClient,
        service_name: str,
        ip: Optional[str] = None,
        port: int = 8000,
        group: str = "DEFAULT_GROUP",
        cluster_name: str = "DEFAULT",
        weight: float = 1.0,
        metadata: dict = None,
        heartbeat_interval: int = 5,  # seconds
    ):
        self.client = client
        self.service_name = service_name
        self.ip = ip or get_local_ip()
        self.port = port
        self.group = group
        self.cluster_name = cluster_name
        self.weight = weight
        self.metadata = metadata or {}
        self.heartbeat_interval = heartbeat_interval
        self._registered = False
        self._shutdown = False
        self._lock = threading.Lock()
        self._heartbeat_timer: Optional[SafeTimer] = None

    def register(self):
        """注册服务实例"""
        with self._lock:
            if self._registered or self._shutdown:
                return
            try:
                self.client.add_naming_instance(
                    service_name=self.service_name,
                    ip=self.ip,
                    port=self.port,
                    group_name=self.group,
                    cluster_name=self.cluster_name,
                    weight=self.weight,
                    metadata=self.metadata,
                    ephemeral=True,  # 临时实例（依赖心跳）
                )
                self._registered = True
                logger.info(
                    f"Service registered: {self.service_name} @ {self.ip}:{self.port}"
                )
                self._start_heartbeat()
            except Exception as e:
                logger.error(f"Failed to register service: {e}")
                raise

    def _send_heartbeat(self):
        """发送心跳（Nacos SDK 内部已支持，但显式调用更可靠）"""
        if not self._registered or self._shutdown:
            return
        try:
            self.client.send_heartbeat(
                service_name=self.service_name,
                ip=self.ip,
                port=self.port,
                group_name=self.group,
                cluster_name=self.cluster_name,
                weight=self.weight,
                metadata=self.metadata,
            )
            logger.debug(f"Heartbeat sent for {self.service_name}")
        except Exception as e:
            logger.warning(f"Heartbeat failed: {e}")
        finally:
            # 确保定时器继续运行，除非明确停止
            if not self._shutdown and self._registered:
                self._start_heartbeat()

    def _start_heartbeat(self):
        self._heartbeat_timer = SafeTimer(self.heartbeat_interval, self._send_heartbeat)
        self._heartbeat_timer.start()

    def deregister(self):
        """主动注销服务（优雅关闭时调用）"""
        with self._lock:
            if not self._registered:
                return
            self._shutdown = True
            if self._heartbeat_timer:
                self._heartbeat_timer.cancel()
            try:
                self.client.remove_naming_instance(
                    service_name=self.service_name,
                    ip=self.ip,
                    port=self.port,
                    group_name=self.group,
                    cluster_name=self.cluster_name,
                )
                logger.info(f"Service deregistered: {self.service_name}")
            except Exception as e:
                logger.error(f"Failed to deregister: {e}")
            self._registered = False



# client.py
import atexit
import logging
import signal
import threading
from typing import Optional

from nacos import NacosClient

from .config import NacosConfig
from .discovery import ServiceDiscovery
from .entity import SharedConfigEntity
from .registry import ServiceRegistry
from .config_app import AppConfig

logger = logging.getLogger(__name__)


class NacosClientManager:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(
        self,
        server_addresses: str,
        namespace: str = "",
        username: str = None,
        password: str = None,
    ):
        if hasattr(self, "_initialized") and self._initialized:
            return
        self.server_addresses = server_addresses
        self.namespace = namespace

        self.nacos_client = NacosClient(
            server_addresses=server_addresses,
            namespace=namespace,
            username=username,
            password=password,
        )

        self.config_manager: Optional[NacosConfig] = None
        self.registry: Optional[ServiceRegistry] = None
        self.discovery = ServiceDiscovery(self.nacos_client)

        self._setup_shutdown_hooks()
        self._initialized = True

    def init_config(
        self,
        data_id: str,
        group: str = "DEFAULT_GROUP",
        config_env: AppConfig = None,
        shared_configs: list[SharedConfigEntity] = [],
    ) -> NacosConfig:
        self.config_manager = NacosConfig(
            self.nacos_client, data_id, group, config_env, shared_configs
        )
        return self.config_manager

    def register_service(
        self,
        service_name: str,
        port: int,
        ip: str = None,
        group: str = "DEFAULT_GROUP",
        metadata: dict = None,
        heartbeat_interval: int = 5,
    ) -> ServiceRegistry:
        self.registry = ServiceRegistry(
            client=self.nacos_client,
            service_name=service_name,
            ip=ip,
            port=port,
            group=group,
            metadata=metadata,
            heartbeat_interval=heartbeat_interval,
        )
        self.registry.register()
        return self.registry

    def get_discovery(self) -> ServiceDiscovery:
        return self.discovery

    def _setup_shutdown_hooks(self):
        """注册优雅关闭钩子"""

        def _shutdown():
            if self.registry:
                self.registry.deregister()

        atexit.register(_shutdown)
        # 捕获 SIGTERM / SIGINT
        signal.signal(signal.SIGTERM, lambda s, f: _shutdown() or exit(0))
        signal.signal(signal.SIGINT, lambda s, f: _shutdown() or exit(0))

    def close(self):
        if self.registry:
            self.registry.deregister()



# discovery.py
import logging
from typing import Dict, List, Optional

from nacos import NacosClient

logger = logging.getLogger(__name__)


class ServiceDiscovery:
    def __init__(self, client: NacosClient):
        self.client = client

    def get_instances(
        self,
        service_name: str,
        group: str = "DEFAULT_GROUP",
        healthy_only: bool = True,
    ) -> List[Dict]:
        """
        获取服务实例列表
        返回格式: [{"ip": "...", "port": ..., "metadata": {...}, ...}]
        """
        try:
            instances = self.client.list_naming_instance(
                service_name=service_name,
                group_name=group,
                healthy_only=healthy_only,
            )
            # 过滤出可用字段
            result = []
            for inst in instances["hosts"]:
                result.append(
                    {
                        "ip": inst["ip"],
                        "port": inst["port"],
                        "weight": inst["weight"],
                        "healthy": inst["healthy"],
                        "metadata": getattr(inst, "metadata", {}),
                        "cluster": getattr(inst, "clusterName", "DEFAULT"),
                    }
                )
            logger.debug(f"Discovered {len(result)} instances for {service_name}")
            return result
        except Exception as e:
            logger.error(f"Failed to discover {service_name}: {e}")
            return []

    def get_one_healthy_instance(
        self, service_name: str, group: str = "DEFAULT_GROUP"
    ) -> Optional[Dict]:
        instances = self.get_instances(service_name, group, healthy_only=True)
        return instances[0] if instances else None
